/* eslint-disable camelcase */
import React from 'react';

/**
 * Similar to useCallback, with a few subtle differences:
 * - The returned function is a stable reference, and will always be the same between renders
 * - No dependency lists required
 * - Properties or state accessed within the callback will always be "current"
 */
export function useEvent(callback) {
  // Keep track of the latest callback:
  const latestRef = React.useRef(callback);
  latestRef.current = callback; // Create a stable callback that always calls the latest callback:
  // using useRef instead of useCallback avoids creating and empty array on every render

  const stableRef = React.useRef(null);

  if (!stableRef.current) {
    stableRef.current = function () {
      return latestRef.current.apply(this, arguments);
    };
  }

  return stableRef.current;
}
export default useEvent;