import { useMemo, useRef } from 'react';
/**
 * Similar to useMemo, with a few subtle differences:
 * - The factory function has a argument which is the result of the previous call to the factory function
 */

function useMemoWithArgument(factory, deps) {
  const ref = useRef({
    result: undefined,
    deps: undefined,
    extra: {}
  });
  return useMemo(() => {
    const result = factory(ref.current.result, ref.current.deps, ref.current.extra);
    ref.current.result = result;
    ref.current.deps = deps;
    return result; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, deps);
}

export default useMemoWithArgument;