import { useMemo, useState } from 'react';
import useEvent from './use-event';

function refsInitialState() {
  return {
    proms: []
  };
}
/**
 * Similar to useState, with a few subtle differences:
 * - The returned function is a promise that will resolve when FC is recalled and the state is same with the update value;
 */


function useStateWithPromise(initialState) {
  const [state, setState] = useState(initialState);
  const [$refs] = useState(refsInitialState);
  useMemo(() => {
    if (!$refs.proms.length) return;
    const proms = $refs.proms.splice(0, $refs.proms.length);
    proms.forEach(prom => {
      if (prom.options.alwaysResolve || prom.nextState === state) {
        prom.resolve(state);
      } else if (process.env.NODE_ENV !== 'production') {
        console.error('[useStateWithPromise warning]: the state has been updated, but its value is different from the updated value', {
          state,
          updateState: prom.nextState
        });
      }
    });
  }, [$refs, state]);
  const setStateWithPromise = useEvent((stateAction, options = {}) => {
    let prom = {
      options
    };
    const ret = new Promise((resolve, reject) => {
      prom.resolve = resolve;
      prom.reject = reject;
    });

    if (typeof stateAction === 'function') {
      setState(prevState => {
        prom.nextState = stateAction(prevState);
        if (prevState === prom.nextState) prom.resolve(prevState);else $refs.proms.push(prom);
        return prom.nextState;
      });
    } else {
      if (stateAction === state) prom.resolve(state);else {
        $refs.proms.push(prom);
        setState(prom.nextState = stateAction);
      }
    }

    return ret;
  });
  return [state, setStateWithPromise];
}

export default useStateWithPromise;