"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useEvent = useEvent;
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable camelcase */

/**
 * Similar to useCallback, with a few subtle differences:
 * - The returned function is a stable reference, and will always be the same between renders
 * - No dependency lists required
 * - Properties or state accessed within the callback will always be "current"
 */
function useEvent(callback) {
  // Keep track of the latest callback:
  var latestRef = _react.default.useRef(callback);

  latestRef.current = callback; // Create a stable callback that always calls the latest callback:
  // using useRef instead of useCallback avoids creating and empty array on every render

  var stableRef = _react.default.useRef(null);

  if (!stableRef.current) {
    stableRef.current = function () {
      return latestRef.current.apply(this, arguments);
    };
  }

  return stableRef.current;
}

var _default = useEvent;
exports.default = _default;
//# sourceMappingURL=use-event.js.map